#include "ADC.h"
#include "ADCDefs.h"
#include "DataOut.h"

CY_ISR_PROTO(Drdy1);
CY_ISR_PROTO(Drdy2);
CY_ISR_PROTO(DRx1);
CY_ISR_PROTO(DRx2);

uint8_t Buffer1[21] = {0};
uint8_t Buffer2[21] = {0};
volatile uint8_t Ptr1 = 0;
volatile uint8_t Ptr2 = 0;
volatile uint8_t phase = 0;

void ADC_WriteReg(uint8_t dev, uint8_t reg, uint8_t val)
{
    if(dev==1)
    {
        Adc0_WriteTxData(ADC_WRITEREG | reg);
        Adc0_WriteTxData(val);
        while(Adc0_GetTxBufferSize > 0);
    }
    else if(dev==2)
    {
        Adc1_WriteTxData(ADC_WRITEREG | reg);
        Adc1_WriteTxData(val);
        while(Adc1_GetTxBufferSize > 0);
    }
    CyDelayUs(10);
}

void ADC_SendReadCommand(uint8_t dev)
{
    if(dev==1)
    {
        Adc0_WriteTxData(0x80);
        Adc0_WriteTxData(0x00);
        Adc0_WriteTxData(0x00);
    }
    else if(dev==2)
    {
        Adc1_WriteTxData(0x80);
        Adc1_WriteTxData(0x00);
        Adc1_WriteTxData(0x00);
    }
}


void ADC_Init()
{
    Reset1_Write(0);
    Reset2_Write(0);
    CyDelay(1);
    Adc0_Start();
    Adc1_Start();
    DataReady1_StartEx(Drdy1);
    DataReady2_StartEx(Drdy2);
    DataReceived1_StartEx(DRx1);
    DataReceived2_StartEx(DRx2);
    Reset1_Write(1);
    Reset2_Write(1);
    CyDelay(10);
 
    ADC_WriteReg(1,ADC_REG_CHDIS,ADC_CHDIS_CH2 | ADC_CHDIS_CH3 | ADC_CHDIS_CH7);
    ADC_WriteReg(1,ADC_REG_CFG1,ADC_CFG1_PWRMODE);
    ADC_WriteReg(1,ADC_REG_CFG3,ADC_CFG3_CONVSTDEGLITCH_DIS);
    ADC_WriteReg(1,ADC_REG_DOUTFMT,ADC_DOUTFMT_1LINE | ADC_DOUTFMT_DIV4);
    ADC_WriteReg(1,ADC_REG_SRCNMSB,0x00);
    ADC_WriteReg(1,ADC_REG_SRCNLSB,0xFA);
    ADC_WriteReg(1,ADC_REG_SRCIFMSB,0x00);
    ADC_WriteReg(1,ADC_REG_SRCIFLSB,0x00);
    ADC_WriteReg(1,ADC_REG_SRCUPD,ADC_SRCUPD_LOADUPD);
    ADC_WriteReg(1,ADC_REG_SRCUPD,0);
    ADC_WriteReg(2,ADC_REG_CHDIS,ADC_CHDIS_CH2 | ADC_CHDIS_CH3 | ADC_CHDIS_CH7);
    ADC_WriteReg(2,ADC_REG_CFG1,ADC_CFG1_PWRMODE);
    ADC_WriteReg(2,ADC_REG_CFG3,ADC_CFG3_CONVSTDEGLITCH_DIS);
    ADC_WriteReg(2,ADC_REG_DOUTFMT,ADC_DOUTFMT_1LINE | ADC_DOUTFMT_DIV4);
    ADC_WriteReg(2,ADC_REG_SRCNMSB,0x00);
    ADC_WriteReg(2,ADC_REG_SRCNLSB,0xFA);
    ADC_WriteReg(2,ADC_REG_SRCIFMSB,0x00);
    ADC_WriteReg(2,ADC_REG_SRCIFLSB,0x00);
    ADC_WriteReg(2,ADC_REG_SRCUPD,ADC_SRCUPD_LOADUPD);
    ADC_WriteReg(2,ADC_REG_SRCUPD,0);
    CyDelay(1);
    
    ADC_WriteReg(1,ADC_REG_CFG3,ADC_CFG3_CONVSTDEGLITCH_DIS | ADC_CFG3_SPISLAVEMODEEN);
    ADC_WriteReg(2,ADC_REG_CFG3,ADC_CFG3_CONVSTDEGLITCH_DIS | ADC_CFG3_SPISLAVEMODEEN);
    CyDelay(1);

    Adc0_ClearFIFO();
    Adc1_ClearFIFO();
    CyAdcSynchroClock_Stop();
    phase = 1;
    ADC_SendReadCommand(1);
    ADC_SendReadCommand(2);
}

CY_ISR(Drdy1)
{
    Ptr1 = 0;
    ADC_SendReadCommand(1);
}
CY_ISR(Drdy2)
{
    Ptr2 = 0;
    ADC_SendReadCommand(2);
}
CY_ISR(DRx1)
{
    if(phase)
    {
        Buffer1[Ptr1++] = Adc0_ReadRxData();
        Buffer1[Ptr1++] = Adc0_ReadRxData();
        Buffer1[Ptr1++] = Adc0_ReadRxData();
        CyDelayUs(1);
        if(Ptr1 < 21)
            ADC_SendReadCommand(1);
        if(Ptr1 == 6)
            DOut_DataReady(0,(uint32_t)Buffer1[1] << 16 | (uint32_t)Buffer1[2] << 8 | Buffer1[3]); 
        else if(Ptr1 == 9)
            DOut_DataReady(1,(uint32_t)Buffer1[5] << 16 | (uint32_t)Buffer1[6] << 8 | Buffer1[7]);
        else if(Ptr1 == 12)
            DOut_DataReady(2,(uint32_t)Buffer1[9] << 16 | (uint32_t)Buffer1[10] << 8 | Buffer1[11]);
        else if(Ptr1 == 18)
            DOut_DataReady(3,(uint32_t)Buffer1[13] << 16 | (uint32_t)Buffer1[14] << 8 | Buffer1[15]);
        else if(Ptr1 == 21)
            DOut_DataReady(4,(uint32_t)Buffer1[17] << 16 | (uint32_t)Buffer1[18] << 8 | Buffer1[19]);
    }
    Adc0_ReadTxStatus();
}
CY_ISR(DRx2)
{
    if(phase)
    {
        Buffer2[Ptr2++] = Adc1_ReadRxData();
        Buffer2[Ptr2++] = Adc1_ReadRxData();
        Buffer2[Ptr2++] = Adc1_ReadRxData();
        CyDelayUs(1);
        if(Ptr2 < 21)
            ADC_SendReadCommand(2);
        if(Ptr2 == 6)
            DOut_DataReady(5,(uint32_t)Buffer2[1] << 16 | (uint32_t)Buffer2[2] << 8 | Buffer2[3]); 
        else if(Ptr2 == 9)
            DOut_DataReady(6,(uint32_t)Buffer2[5] << 16 | (uint32_t)Buffer2[6] << 8 | Buffer2[7]);
        else if(Ptr2 == 12)
            DOut_DataReady(7,(uint32_t)Buffer2[9] << 16 | (uint32_t)Buffer2[10] << 8 | Buffer2[11]);
        else if(Ptr2 == 18)
            DOut_DataReady(8,(uint32_t)Buffer2[13] << 16 | (uint32_t)Buffer2[14] << 8 | Buffer2[15]);
        else if(Ptr2 == 21)
            DOut_DataReady(9,(uint32_t)Buffer2[17] << 16 | (uint32_t)Buffer2[18] << 8 | Buffer2[19]);
    }
    Adc1_ReadTxStatus();
}