/*******************************************************************************
* File Name: Adc0.h
* Version 2.50
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_Adc0_H)
#define CY_SPIM_Adc0_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define Adc0_INTERNAL_CLOCK             (0u)

#if(0u != Adc0_INTERNAL_CLOCK)
    #include "Adc0_IntClock.h"
#endif /* (0u != Adc0_INTERNAL_CLOCK) */

#define Adc0_MODE                       (4u)
#define Adc0_DATA_WIDTH                 (8u)
#define Adc0_MODE_USE_ZERO              (0u)
#define Adc0_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define Adc0_TX_BUFFER_SIZE             (4u)
#define Adc0_RX_BUFFER_SIZE             (4u)
#define Adc0_INTERNAL_TX_INT_ENABLED    (0u)
#define Adc0_INTERNAL_RX_INT_ENABLED    (0u)

#define Adc0_SINGLE_REG_SIZE            (8u)
#define Adc0_USE_SECOND_DATAPATH        (Adc0_DATA_WIDTH > Adc0_SINGLE_REG_SIZE)

#define Adc0_FIFO_SIZE                  (4u)
#define Adc0_TX_SOFTWARE_BUF_ENABLED    ((0u != Adc0_INTERNAL_TX_INT_ENABLED) && \
                                                     (Adc0_TX_BUFFER_SIZE > Adc0_FIFO_SIZE))

#define Adc0_RX_SOFTWARE_BUF_ENABLED    ((0u != Adc0_INTERNAL_RX_INT_ENABLED) && \
                                                     (Adc0_RX_BUFFER_SIZE > Adc0_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} Adc0_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  Adc0_Init(void)                           ;
void  Adc0_Enable(void)                         ;
void  Adc0_Start(void)                          ;
void  Adc0_Stop(void)                           ;

void  Adc0_EnableTxInt(void)                    ;
void  Adc0_EnableRxInt(void)                    ;
void  Adc0_DisableTxInt(void)                   ;
void  Adc0_DisableRxInt(void)                   ;

void  Adc0_Sleep(void)                          ;
void  Adc0_Wakeup(void)                         ;
void  Adc0_SaveConfig(void)                     ;
void  Adc0_RestoreConfig(void)                  ;

void  Adc0_SetTxInterruptMode(uint8 intSrc)     ;
void  Adc0_SetRxInterruptMode(uint8 intSrc)     ;
uint8 Adc0_ReadTxStatus(void)                   ;
uint8 Adc0_ReadRxStatus(void)                   ;
void  Adc0_WriteTxData(uint8 txData)  \
                                                            ;
uint8 Adc0_ReadRxData(void) \
                                                            ;
uint8 Adc0_GetRxBufferSize(void)                ;
uint8 Adc0_GetTxBufferSize(void)                ;
void  Adc0_ClearRxBuffer(void)                  ;
void  Adc0_ClearTxBuffer(void)                  ;
void  Adc0_ClearFIFO(void)                              ;
void  Adc0_PutArray(const uint8 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != Adc0_BIDIRECTIONAL_MODE)
    void  Adc0_TxEnable(void)                   ;
    void  Adc0_TxDisable(void)                  ;
#endif /* (0u != Adc0_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(Adc0_TX_ISR);
CY_ISR_PROTO(Adc0_RX_ISR);


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 Adc0_initVar;


/***************************************
*           API Constants
***************************************/

#define Adc0_TX_ISR_NUMBER     ((uint8) (Adc0_TxInternalInterrupt__INTC_NUMBER))
#define Adc0_RX_ISR_NUMBER     ((uint8) (Adc0_RxInternalInterrupt__INTC_NUMBER))

#define Adc0_TX_ISR_PRIORITY   ((uint8) (Adc0_TxInternalInterrupt__INTC_PRIOR_NUM))
#define Adc0_RX_ISR_PRIORITY   ((uint8) (Adc0_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define Adc0_INT_ON_SPI_DONE    ((uint8) (0u   << Adc0_STS_SPI_DONE_SHIFT))
#define Adc0_INT_ON_TX_EMPTY    ((uint8) (1u   << Adc0_STS_TX_FIFO_EMPTY_SHIFT))
#define Adc0_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           Adc0_STS_TX_FIFO_NOT_FULL_SHIFT))
#define Adc0_INT_ON_BYTE_COMP   ((uint8) (0u  << Adc0_STS_BYTE_COMPLETE_SHIFT))
#define Adc0_INT_ON_SPI_IDLE    ((uint8) (0u   << Adc0_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define Adc0_INT_ON_TX_NOT_FULL_DEF ((Adc0_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (Adc0_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define Adc0_TX_INIT_INTERRUPTS_MASK    (Adc0_INT_ON_SPI_DONE  | \
                                                     Adc0_INT_ON_TX_EMPTY  | \
                                                     Adc0_INT_ON_TX_NOT_FULL_DEF | \
                                                     Adc0_INT_ON_BYTE_COMP | \
                                                     Adc0_INT_ON_SPI_IDLE)

#define Adc0_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          Adc0_STS_RX_FIFO_FULL_SHIFT))
#define Adc0_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          Adc0_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define Adc0_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          Adc0_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define Adc0_RX_INIT_INTERRUPTS_MASK    (Adc0_INT_ON_RX_FULL      | \
                                                     Adc0_INT_ON_RX_NOT_EMPTY | \
                                                     Adc0_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define Adc0_BITCTR_INIT            (((uint8) (Adc0_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define Adc0_TXDATA_REG (* (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F0_REG)
    #define Adc0_TXDATA_PTR (  (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F0_REG)
    #define Adc0_RXDATA_REG (* (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F1_REG)
    #define Adc0_RXDATA_PTR (  (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(Adc0_USE_SECOND_DATAPATH)
        #define Adc0_TXDATA_REG (* (reg16 *) \
                                          Adc0_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define Adc0_TXDATA_PTR (  (reg16 *) \
                                          Adc0_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define Adc0_RXDATA_REG (* (reg16 *) \
                                          Adc0_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
        #define Adc0_RXDATA_PTR (  (reg16 *) \
                                          Adc0_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
    #else
        #define Adc0_TXDATA_REG (* (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F0_REG)
        #define Adc0_TXDATA_PTR (  (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F0_REG)
        #define Adc0_RXDATA_REG (* (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F1_REG)
        #define Adc0_RXDATA_PTR (  (reg8 *) \
                                                Adc0_BSPIM_sR8_Dp_u0__F1_REG)
    #endif /* (Adc0_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define Adc0_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        Adc0_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define Adc0_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        Adc0_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(Adc0_USE_SECOND_DATAPATH)
    #define Adc0_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        Adc0_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define Adc0_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        Adc0_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* (Adc0_USE_SECOND_DATAPATH) */

#define Adc0_COUNTER_PERIOD_REG     (* (reg8 *) Adc0_BSPIM_BitCounter__PERIOD_REG)
#define Adc0_COUNTER_PERIOD_PTR     (  (reg8 *) Adc0_BSPIM_BitCounter__PERIOD_REG)
#define Adc0_COUNTER_CONTROL_REG    (* (reg8 *) Adc0_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define Adc0_COUNTER_CONTROL_PTR    (  (reg8 *) Adc0_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define Adc0_TX_STATUS_REG          (* (reg8 *) Adc0_BSPIM_TxStsReg__STATUS_REG)
#define Adc0_TX_STATUS_PTR          (  (reg8 *) Adc0_BSPIM_TxStsReg__STATUS_REG)
#define Adc0_RX_STATUS_REG          (* (reg8 *) Adc0_BSPIM_RxStsReg__STATUS_REG)
#define Adc0_RX_STATUS_PTR          (  (reg8 *) Adc0_BSPIM_RxStsReg__STATUS_REG)

#define Adc0_CONTROL_REG            (* (reg8 *) \
                                      Adc0_BSPIM_BidirMode_CtrlReg__CONTROL_REG)
#define Adc0_CONTROL_PTR            (  (reg8 *) \
                                      Adc0_BSPIM_BidirMode_CtrlReg__CONTROL_REG)

#define Adc0_TX_STATUS_MASK_REG     (* (reg8 *) Adc0_BSPIM_TxStsReg__MASK_REG)
#define Adc0_TX_STATUS_MASK_PTR     (  (reg8 *) Adc0_BSPIM_TxStsReg__MASK_REG)
#define Adc0_RX_STATUS_MASK_REG     (* (reg8 *) Adc0_BSPIM_RxStsReg__MASK_REG)
#define Adc0_RX_STATUS_MASK_PTR     (  (reg8 *) Adc0_BSPIM_RxStsReg__MASK_REG)

#define Adc0_TX_STATUS_ACTL_REG     (* (reg8 *) Adc0_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define Adc0_TX_STATUS_ACTL_PTR     (  (reg8 *) Adc0_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define Adc0_RX_STATUS_ACTL_REG     (* (reg8 *) Adc0_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define Adc0_RX_STATUS_ACTL_PTR     (  (reg8 *) Adc0_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(Adc0_USE_SECOND_DATAPATH)
    #define Adc0_AUX_CONTROLDP1     (Adc0_AUX_CONTROL_DP1_REG)
#endif /* (Adc0_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define Adc0_STS_SPI_DONE_SHIFT             (0x00u)
#define Adc0_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define Adc0_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define Adc0_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define Adc0_STS_SPI_IDLE_SHIFT             (0x04u)
#define Adc0_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define Adc0_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define Adc0_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define Adc0_STS_SPI_DONE           ((uint8) (0x01u << Adc0_STS_SPI_DONE_SHIFT))
#define Adc0_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << Adc0_STS_TX_FIFO_EMPTY_SHIFT))
#define Adc0_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << Adc0_STS_TX_FIFO_NOT_FULL_SHIFT))
#define Adc0_STS_BYTE_COMPLETE      ((uint8) (0x01u << Adc0_STS_BYTE_COMPLETE_SHIFT))
#define Adc0_STS_SPI_IDLE           ((uint8) (0x01u << Adc0_STS_SPI_IDLE_SHIFT))
#define Adc0_STS_RX_FIFO_FULL       ((uint8) (0x01u << Adc0_STS_RX_FIFO_FULL_SHIFT))
#define Adc0_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << Adc0_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define Adc0_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << Adc0_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define Adc0_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define Adc0_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define Adc0_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define Adc0_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define Adc0_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define Adc0_FIFO_CLR       (Adc0_TX_FIFO_CLR | Adc0_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define Adc0_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define Adc0_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define Adc0_AUX_CTRL_FIFO0_CLR         (0x01u)
#define Adc0_AUX_CTRL_FIFO1_CLR         (0x02u)
#define Adc0_AUX_CTRL_FIFO0_LVL         (0x04u)
#define Adc0_AUX_CTRL_FIFO1_LVL         (0x08u)
#define Adc0_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define Adc0_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define Adc0_IS_ENABLED (0u != (Adc0_TX_STATUS_ACTL_REG & Adc0_INT_ENABLE))

/* Retuns TX status register */
#define Adc0_GET_STATUS_TX(swTxSts) ( (uint8)(Adc0_TX_STATUS_REG | \
                                                          ((swTxSts) & Adc0_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define Adc0_GET_STATUS_RX(swRxSts) ( (uint8)(Adc0_RX_STATUS_REG | \
                                                          ((swRxSts) & Adc0_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define Adc0_WriteByte   Adc0_WriteTxData
#define Adc0_ReadByte    Adc0_ReadRxData
void  Adc0_SetInterruptMode(uint8 intSrc)       ;
uint8 Adc0_ReadStatus(void)                     ;
void  Adc0_EnableInt(void)                      ;
void  Adc0_DisableInt(void)                     ;

#define Adc0_TXDATA                 (Adc0_TXDATA_REG)
#define Adc0_RXDATA                 (Adc0_RXDATA_REG)
#define Adc0_AUX_CONTROLDP0         (Adc0_AUX_CONTROL_DP0_REG)
#define Adc0_TXBUFFERREAD           (Adc0_txBufferRead)
#define Adc0_TXBUFFERWRITE          (Adc0_txBufferWrite)
#define Adc0_RXBUFFERREAD           (Adc0_rxBufferRead)
#define Adc0_RXBUFFERWRITE          (Adc0_rxBufferWrite)

#define Adc0_COUNTER_PERIOD         (Adc0_COUNTER_PERIOD_REG)
#define Adc0_COUNTER_CONTROL        (Adc0_COUNTER_CONTROL_REG)
#define Adc0_STATUS                 (Adc0_TX_STATUS_REG)
#define Adc0_CONTROL                (Adc0_CONTROL_REG)
#define Adc0_STATUS_MASK            (Adc0_TX_STATUS_MASK_REG)
#define Adc0_STATUS_ACTL            (Adc0_TX_STATUS_ACTL_REG)

#define Adc0_INIT_INTERRUPTS_MASK  (Adc0_INT_ON_SPI_DONE     | \
                                                Adc0_INT_ON_TX_EMPTY     | \
                                                Adc0_INT_ON_TX_NOT_FULL_DEF  | \
                                                Adc0_INT_ON_RX_FULL      | \
                                                Adc0_INT_ON_RX_NOT_EMPTY | \
                                                Adc0_INT_ON_RX_OVER      | \
                                                Adc0_INT_ON_BYTE_COMP)
                                                
#define Adc0_DataWidth                  (Adc0_DATA_WIDTH)
#define Adc0_InternalClockUsed          (Adc0_INTERNAL_CLOCK)
#define Adc0_InternalTxInterruptEnabled (Adc0_INTERNAL_TX_INT_ENABLED)
#define Adc0_InternalRxInterruptEnabled (Adc0_INTERNAL_RX_INT_ENABLED)
#define Adc0_ModeUseZero                (Adc0_MODE_USE_ZERO)
#define Adc0_BidirectionalMode          (Adc0_BIDIRECTIONAL_MODE)
#define Adc0_Mode                       (Adc0_MODE)
#define Adc0_DATAWIDHT                  (Adc0_DATA_WIDTH)
#define Adc0_InternalInterruptEnabled   (0u)

#define Adc0_TXBUFFERSIZE   (Adc0_TX_BUFFER_SIZE)
#define Adc0_RXBUFFERSIZE   (Adc0_RX_BUFFER_SIZE)

#define Adc0_TXBUFFER       Adc0_txBuffer
#define Adc0_RXBUFFER       Adc0_rxBuffer

#endif /* (CY_SPIM_Adc0_H) */


/* [] END OF FILE */
