/*******************************************************************************
* File Name: Adc0_PM.c
* Version 2.50
*
* Description:
*  This file contains the setup, control and status commands to support
*  component operations in low power mode.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Adc0_PVT.h"

static Adc0_BACKUP_STRUCT Adc0_backup =
{
    Adc0_DISABLED,
    Adc0_BITCTR_INIT,
};


/*******************************************************************************
* Function Name: Adc0_SaveConfig
********************************************************************************
*
* Summary:
*  Empty function. Included for consistency with other components.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Adc0_SaveConfig(void) 
{

}


/*******************************************************************************
* Function Name: Adc0_RestoreConfig
********************************************************************************
*
* Summary:
*  Empty function. Included for consistency with other components.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Adc0_RestoreConfig(void) 
{

}


/*******************************************************************************
* Function Name: Adc0_Sleep
********************************************************************************
*
* Summary:
*  Prepare SPIM Component goes to sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Adc0_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Adc0_Sleep(void) 
{
    /* Save components enable state */
    Adc0_backup.enableState = ((uint8) Adc0_IS_ENABLED);

    Adc0_Stop();
}


/*******************************************************************************
* Function Name: Adc0_Wakeup
********************************************************************************
*
* Summary:
*  Prepare SPIM Component to wake up.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Adc0_backup - used when non-retention registers are restored.
*  Adc0_txBufferWrite - modified every function call - resets to
*  zero.
*  Adc0_txBufferRead - modified every function call - resets to
*  zero.
*  Adc0_rxBufferWrite - modified every function call - resets to
*  zero.
*  Adc0_rxBufferRead - modified every function call - resets to
*  zero.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Adc0_Wakeup(void) 
{
    #if(Adc0_RX_SOFTWARE_BUF_ENABLED)
        Adc0_rxBufferFull  = 0u;
        Adc0_rxBufferRead  = 0u;
        Adc0_rxBufferWrite = 0u;
    #endif /* (Adc0_RX_SOFTWARE_BUF_ENABLED) */

    #if(Adc0_TX_SOFTWARE_BUF_ENABLED)
        Adc0_txBufferFull  = 0u;
        Adc0_txBufferRead  = 0u;
        Adc0_txBufferWrite = 0u;
    #endif /* (Adc0_TX_SOFTWARE_BUF_ENABLED) */

    /* Clear any data from the RX and TX FIFO */
    Adc0_ClearFIFO();

    /* Restore components block enable state */
    if(0u != Adc0_backup.enableState)
    {
        Adc0_Enable();
    }
}


/* [] END OF FILE */
