/*******************************************************************************
* File Name: Adc1.h
* Version 2.50
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_Adc1_H)
#define CY_SPIM_Adc1_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define Adc1_INTERNAL_CLOCK             (0u)

#if(0u != Adc1_INTERNAL_CLOCK)
    #include "Adc1_IntClock.h"
#endif /* (0u != Adc1_INTERNAL_CLOCK) */

#define Adc1_MODE                       (4u)
#define Adc1_DATA_WIDTH                 (8u)
#define Adc1_MODE_USE_ZERO              (0u)
#define Adc1_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define Adc1_TX_BUFFER_SIZE             (4u)
#define Adc1_RX_BUFFER_SIZE             (4u)
#define Adc1_INTERNAL_TX_INT_ENABLED    (0u)
#define Adc1_INTERNAL_RX_INT_ENABLED    (0u)

#define Adc1_SINGLE_REG_SIZE            (8u)
#define Adc1_USE_SECOND_DATAPATH        (Adc1_DATA_WIDTH > Adc1_SINGLE_REG_SIZE)

#define Adc1_FIFO_SIZE                  (4u)
#define Adc1_TX_SOFTWARE_BUF_ENABLED    ((0u != Adc1_INTERNAL_TX_INT_ENABLED) && \
                                                     (Adc1_TX_BUFFER_SIZE > Adc1_FIFO_SIZE))

#define Adc1_RX_SOFTWARE_BUF_ENABLED    ((0u != Adc1_INTERNAL_RX_INT_ENABLED) && \
                                                     (Adc1_RX_BUFFER_SIZE > Adc1_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} Adc1_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  Adc1_Init(void)                           ;
void  Adc1_Enable(void)                         ;
void  Adc1_Start(void)                          ;
void  Adc1_Stop(void)                           ;

void  Adc1_EnableTxInt(void)                    ;
void  Adc1_EnableRxInt(void)                    ;
void  Adc1_DisableTxInt(void)                   ;
void  Adc1_DisableRxInt(void)                   ;

void  Adc1_Sleep(void)                          ;
void  Adc1_Wakeup(void)                         ;
void  Adc1_SaveConfig(void)                     ;
void  Adc1_RestoreConfig(void)                  ;

void  Adc1_SetTxInterruptMode(uint8 intSrc)     ;
void  Adc1_SetRxInterruptMode(uint8 intSrc)     ;
uint8 Adc1_ReadTxStatus(void)                   ;
uint8 Adc1_ReadRxStatus(void)                   ;
void  Adc1_WriteTxData(uint8 txData)  \
                                                            ;
uint8 Adc1_ReadRxData(void) \
                                                            ;
uint8 Adc1_GetRxBufferSize(void)                ;
uint8 Adc1_GetTxBufferSize(void)                ;
void  Adc1_ClearRxBuffer(void)                  ;
void  Adc1_ClearTxBuffer(void)                  ;
void  Adc1_ClearFIFO(void)                              ;
void  Adc1_PutArray(const uint8 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != Adc1_BIDIRECTIONAL_MODE)
    void  Adc1_TxEnable(void)                   ;
    void  Adc1_TxDisable(void)                  ;
#endif /* (0u != Adc1_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(Adc1_TX_ISR);
CY_ISR_PROTO(Adc1_RX_ISR);


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 Adc1_initVar;


/***************************************
*           API Constants
***************************************/

#define Adc1_TX_ISR_NUMBER     ((uint8) (Adc1_TxInternalInterrupt__INTC_NUMBER))
#define Adc1_RX_ISR_NUMBER     ((uint8) (Adc1_RxInternalInterrupt__INTC_NUMBER))

#define Adc1_TX_ISR_PRIORITY   ((uint8) (Adc1_TxInternalInterrupt__INTC_PRIOR_NUM))
#define Adc1_RX_ISR_PRIORITY   ((uint8) (Adc1_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define Adc1_INT_ON_SPI_DONE    ((uint8) (0u   << Adc1_STS_SPI_DONE_SHIFT))
#define Adc1_INT_ON_TX_EMPTY    ((uint8) (1u   << Adc1_STS_TX_FIFO_EMPTY_SHIFT))
#define Adc1_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           Adc1_STS_TX_FIFO_NOT_FULL_SHIFT))
#define Adc1_INT_ON_BYTE_COMP   ((uint8) (0u  << Adc1_STS_BYTE_COMPLETE_SHIFT))
#define Adc1_INT_ON_SPI_IDLE    ((uint8) (0u   << Adc1_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define Adc1_INT_ON_TX_NOT_FULL_DEF ((Adc1_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (Adc1_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define Adc1_TX_INIT_INTERRUPTS_MASK    (Adc1_INT_ON_SPI_DONE  | \
                                                     Adc1_INT_ON_TX_EMPTY  | \
                                                     Adc1_INT_ON_TX_NOT_FULL_DEF | \
                                                     Adc1_INT_ON_BYTE_COMP | \
                                                     Adc1_INT_ON_SPI_IDLE)

#define Adc1_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          Adc1_STS_RX_FIFO_FULL_SHIFT))
#define Adc1_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          Adc1_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define Adc1_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          Adc1_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define Adc1_RX_INIT_INTERRUPTS_MASK    (Adc1_INT_ON_RX_FULL      | \
                                                     Adc1_INT_ON_RX_NOT_EMPTY | \
                                                     Adc1_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define Adc1_BITCTR_INIT            (((uint8) (Adc1_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define Adc1_TXDATA_REG (* (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F0_REG)
    #define Adc1_TXDATA_PTR (  (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F0_REG)
    #define Adc1_RXDATA_REG (* (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F1_REG)
    #define Adc1_RXDATA_PTR (  (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(Adc1_USE_SECOND_DATAPATH)
        #define Adc1_TXDATA_REG (* (reg16 *) \
                                          Adc1_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define Adc1_TXDATA_PTR (  (reg16 *) \
                                          Adc1_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define Adc1_RXDATA_REG (* (reg16 *) \
                                          Adc1_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
        #define Adc1_RXDATA_PTR (  (reg16 *) \
                                          Adc1_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
    #else
        #define Adc1_TXDATA_REG (* (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F0_REG)
        #define Adc1_TXDATA_PTR (  (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F0_REG)
        #define Adc1_RXDATA_REG (* (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F1_REG)
        #define Adc1_RXDATA_PTR (  (reg8 *) \
                                                Adc1_BSPIM_sR8_Dp_u0__F1_REG)
    #endif /* (Adc1_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define Adc1_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        Adc1_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define Adc1_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        Adc1_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(Adc1_USE_SECOND_DATAPATH)
    #define Adc1_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        Adc1_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define Adc1_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        Adc1_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* (Adc1_USE_SECOND_DATAPATH) */

#define Adc1_COUNTER_PERIOD_REG     (* (reg8 *) Adc1_BSPIM_BitCounter__PERIOD_REG)
#define Adc1_COUNTER_PERIOD_PTR     (  (reg8 *) Adc1_BSPIM_BitCounter__PERIOD_REG)
#define Adc1_COUNTER_CONTROL_REG    (* (reg8 *) Adc1_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define Adc1_COUNTER_CONTROL_PTR    (  (reg8 *) Adc1_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define Adc1_TX_STATUS_REG          (* (reg8 *) Adc1_BSPIM_TxStsReg__STATUS_REG)
#define Adc1_TX_STATUS_PTR          (  (reg8 *) Adc1_BSPIM_TxStsReg__STATUS_REG)
#define Adc1_RX_STATUS_REG          (* (reg8 *) Adc1_BSPIM_RxStsReg__STATUS_REG)
#define Adc1_RX_STATUS_PTR          (  (reg8 *) Adc1_BSPIM_RxStsReg__STATUS_REG)

#define Adc1_CONTROL_REG            (* (reg8 *) \
                                      Adc1_BSPIM_BidirMode_CtrlReg__CONTROL_REG)
#define Adc1_CONTROL_PTR            (  (reg8 *) \
                                      Adc1_BSPIM_BidirMode_CtrlReg__CONTROL_REG)

#define Adc1_TX_STATUS_MASK_REG     (* (reg8 *) Adc1_BSPIM_TxStsReg__MASK_REG)
#define Adc1_TX_STATUS_MASK_PTR     (  (reg8 *) Adc1_BSPIM_TxStsReg__MASK_REG)
#define Adc1_RX_STATUS_MASK_REG     (* (reg8 *) Adc1_BSPIM_RxStsReg__MASK_REG)
#define Adc1_RX_STATUS_MASK_PTR     (  (reg8 *) Adc1_BSPIM_RxStsReg__MASK_REG)

#define Adc1_TX_STATUS_ACTL_REG     (* (reg8 *) Adc1_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define Adc1_TX_STATUS_ACTL_PTR     (  (reg8 *) Adc1_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define Adc1_RX_STATUS_ACTL_REG     (* (reg8 *) Adc1_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define Adc1_RX_STATUS_ACTL_PTR     (  (reg8 *) Adc1_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(Adc1_USE_SECOND_DATAPATH)
    #define Adc1_AUX_CONTROLDP1     (Adc1_AUX_CONTROL_DP1_REG)
#endif /* (Adc1_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define Adc1_STS_SPI_DONE_SHIFT             (0x00u)
#define Adc1_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define Adc1_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define Adc1_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define Adc1_STS_SPI_IDLE_SHIFT             (0x04u)
#define Adc1_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define Adc1_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define Adc1_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define Adc1_STS_SPI_DONE           ((uint8) (0x01u << Adc1_STS_SPI_DONE_SHIFT))
#define Adc1_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << Adc1_STS_TX_FIFO_EMPTY_SHIFT))
#define Adc1_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << Adc1_STS_TX_FIFO_NOT_FULL_SHIFT))
#define Adc1_STS_BYTE_COMPLETE      ((uint8) (0x01u << Adc1_STS_BYTE_COMPLETE_SHIFT))
#define Adc1_STS_SPI_IDLE           ((uint8) (0x01u << Adc1_STS_SPI_IDLE_SHIFT))
#define Adc1_STS_RX_FIFO_FULL       ((uint8) (0x01u << Adc1_STS_RX_FIFO_FULL_SHIFT))
#define Adc1_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << Adc1_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define Adc1_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << Adc1_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define Adc1_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define Adc1_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define Adc1_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define Adc1_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define Adc1_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define Adc1_FIFO_CLR       (Adc1_TX_FIFO_CLR | Adc1_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define Adc1_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define Adc1_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define Adc1_AUX_CTRL_FIFO0_CLR         (0x01u)
#define Adc1_AUX_CTRL_FIFO1_CLR         (0x02u)
#define Adc1_AUX_CTRL_FIFO0_LVL         (0x04u)
#define Adc1_AUX_CTRL_FIFO1_LVL         (0x08u)
#define Adc1_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define Adc1_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define Adc1_IS_ENABLED (0u != (Adc1_TX_STATUS_ACTL_REG & Adc1_INT_ENABLE))

/* Retuns TX status register */
#define Adc1_GET_STATUS_TX(swTxSts) ( (uint8)(Adc1_TX_STATUS_REG | \
                                                          ((swTxSts) & Adc1_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define Adc1_GET_STATUS_RX(swRxSts) ( (uint8)(Adc1_RX_STATUS_REG | \
                                                          ((swRxSts) & Adc1_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define Adc1_WriteByte   Adc1_WriteTxData
#define Adc1_ReadByte    Adc1_ReadRxData
void  Adc1_SetInterruptMode(uint8 intSrc)       ;
uint8 Adc1_ReadStatus(void)                     ;
void  Adc1_EnableInt(void)                      ;
void  Adc1_DisableInt(void)                     ;

#define Adc1_TXDATA                 (Adc1_TXDATA_REG)
#define Adc1_RXDATA                 (Adc1_RXDATA_REG)
#define Adc1_AUX_CONTROLDP0         (Adc1_AUX_CONTROL_DP0_REG)
#define Adc1_TXBUFFERREAD           (Adc1_txBufferRead)
#define Adc1_TXBUFFERWRITE          (Adc1_txBufferWrite)
#define Adc1_RXBUFFERREAD           (Adc1_rxBufferRead)
#define Adc1_RXBUFFERWRITE          (Adc1_rxBufferWrite)

#define Adc1_COUNTER_PERIOD         (Adc1_COUNTER_PERIOD_REG)
#define Adc1_COUNTER_CONTROL        (Adc1_COUNTER_CONTROL_REG)
#define Adc1_STATUS                 (Adc1_TX_STATUS_REG)
#define Adc1_CONTROL                (Adc1_CONTROL_REG)
#define Adc1_STATUS_MASK            (Adc1_TX_STATUS_MASK_REG)
#define Adc1_STATUS_ACTL            (Adc1_TX_STATUS_ACTL_REG)

#define Adc1_INIT_INTERRUPTS_MASK  (Adc1_INT_ON_SPI_DONE     | \
                                                Adc1_INT_ON_TX_EMPTY     | \
                                                Adc1_INT_ON_TX_NOT_FULL_DEF  | \
                                                Adc1_INT_ON_RX_FULL      | \
                                                Adc1_INT_ON_RX_NOT_EMPTY | \
                                                Adc1_INT_ON_RX_OVER      | \
                                                Adc1_INT_ON_BYTE_COMP)
                                                
#define Adc1_DataWidth                  (Adc1_DATA_WIDTH)
#define Adc1_InternalClockUsed          (Adc1_INTERNAL_CLOCK)
#define Adc1_InternalTxInterruptEnabled (Adc1_INTERNAL_TX_INT_ENABLED)
#define Adc1_InternalRxInterruptEnabled (Adc1_INTERNAL_RX_INT_ENABLED)
#define Adc1_ModeUseZero                (Adc1_MODE_USE_ZERO)
#define Adc1_BidirectionalMode          (Adc1_BIDIRECTIONAL_MODE)
#define Adc1_Mode                       (Adc1_MODE)
#define Adc1_DATAWIDHT                  (Adc1_DATA_WIDTH)
#define Adc1_InternalInterruptEnabled   (0u)

#define Adc1_TXBUFFERSIZE   (Adc1_TX_BUFFER_SIZE)
#define Adc1_RXBUFFERSIZE   (Adc1_RX_BUFFER_SIZE)

#define Adc1_TXBUFFER       Adc1_txBuffer
#define Adc1_RXBUFFER       Adc1_rxBuffer

#endif /* (CY_SPIM_Adc1_H) */


/* [] END OF FILE */
