/*******************************************************************************
* File Name: Adc1_PM.c
* Version 2.50
*
* Description:
*  This file contains the setup, control and status commands to support
*  component operations in low power mode.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Adc1_PVT.h"

static Adc1_BACKUP_STRUCT Adc1_backup =
{
    Adc1_DISABLED,
    Adc1_BITCTR_INIT,
};


/*******************************************************************************
* Function Name: Adc1_SaveConfig
********************************************************************************
*
* Summary:
*  Empty function. Included for consistency with other components.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Adc1_SaveConfig(void) 
{

}


/*******************************************************************************
* Function Name: Adc1_RestoreConfig
********************************************************************************
*
* Summary:
*  Empty function. Included for consistency with other components.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Adc1_RestoreConfig(void) 
{

}


/*******************************************************************************
* Function Name: Adc1_Sleep
********************************************************************************
*
* Summary:
*  Prepare SPIM Component goes to sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Adc1_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Adc1_Sleep(void) 
{
    /* Save components enable state */
    Adc1_backup.enableState = ((uint8) Adc1_IS_ENABLED);

    Adc1_Stop();
}


/*******************************************************************************
* Function Name: Adc1_Wakeup
********************************************************************************
*
* Summary:
*  Prepare SPIM Component to wake up.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Adc1_backup - used when non-retention registers are restored.
*  Adc1_txBufferWrite - modified every function call - resets to
*  zero.
*  Adc1_txBufferRead - modified every function call - resets to
*  zero.
*  Adc1_rxBufferWrite - modified every function call - resets to
*  zero.
*  Adc1_rxBufferRead - modified every function call - resets to
*  zero.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void Adc1_Wakeup(void) 
{
    #if(Adc1_RX_SOFTWARE_BUF_ENABLED)
        Adc1_rxBufferFull  = 0u;
        Adc1_rxBufferRead  = 0u;
        Adc1_rxBufferWrite = 0u;
    #endif /* (Adc1_RX_SOFTWARE_BUF_ENABLED) */

    #if(Adc1_TX_SOFTWARE_BUF_ENABLED)
        Adc1_txBufferFull  = 0u;
        Adc1_txBufferRead  = 0u;
        Adc1_txBufferWrite = 0u;
    #endif /* (Adc1_TX_SOFTWARE_BUF_ENABLED) */

    /* Clear any data from the RX and TX FIFO */
    Adc1_ClearFIFO();

    /* Restore components block enable state */
    if(0u != Adc1_backup.enableState)
    {
        Adc1_Enable();
    }
}


/* [] END OF FILE */
