/*******************************************************************************
* File Name: CyAdcMasterClock.h
* Version 2.20
*
*  Description:
*   Provides the function and constant definitions for the clock component.
*
*  Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CLOCK_CyAdcMasterClock_H)
#define CY_CLOCK_CyAdcMasterClock_H

#include <cytypes.h>
#include <cyfitter.h>


/***************************************
* Conditional Compilation Parameters
***************************************/

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component cy_clock_v2_20 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5LP) */


/***************************************
*        Function Prototypes
***************************************/

void CyAdcMasterClock_Start(void) ;
void CyAdcMasterClock_Stop(void) ;

#if(CY_PSOC3 || CY_PSOC5LP)
void CyAdcMasterClock_StopBlock(void) ;
#endif /* (CY_PSOC3 || CY_PSOC5LP) */

void CyAdcMasterClock_StandbyPower(uint8 state) ;
void CyAdcMasterClock_SetDividerRegister(uint16 clkDivider, uint8 restart) 
                                ;
uint16 CyAdcMasterClock_GetDividerRegister(void) ;
void CyAdcMasterClock_SetModeRegister(uint8 modeBitMask) ;
void CyAdcMasterClock_ClearModeRegister(uint8 modeBitMask) ;
uint8 CyAdcMasterClock_GetModeRegister(void) ;
void CyAdcMasterClock_SetSourceRegister(uint8 clkSource) ;
uint8 CyAdcMasterClock_GetSourceRegister(void) ;
#if defined(CyAdcMasterClock__CFG3)
void CyAdcMasterClock_SetPhaseRegister(uint8 clkPhase) ;
uint8 CyAdcMasterClock_GetPhaseRegister(void) ;
#endif /* defined(CyAdcMasterClock__CFG3) */

#define CyAdcMasterClock_Enable()                       CyAdcMasterClock_Start()
#define CyAdcMasterClock_Disable()                      CyAdcMasterClock_Stop()
#define CyAdcMasterClock_SetDivider(clkDivider)         CyAdcMasterClock_SetDividerRegister(clkDivider, 1u)
#define CyAdcMasterClock_SetDividerValue(clkDivider)    CyAdcMasterClock_SetDividerRegister((clkDivider) - 1u, 1u)
#define CyAdcMasterClock_SetMode(clkMode)               CyAdcMasterClock_SetModeRegister(clkMode)
#define CyAdcMasterClock_SetSource(clkSource)           CyAdcMasterClock_SetSourceRegister(clkSource)
#if defined(CyAdcMasterClock__CFG3)
#define CyAdcMasterClock_SetPhase(clkPhase)             CyAdcMasterClock_SetPhaseRegister(clkPhase)
#define CyAdcMasterClock_SetPhaseValue(clkPhase)        CyAdcMasterClock_SetPhaseRegister((clkPhase) + 1u)
#endif /* defined(CyAdcMasterClock__CFG3) */


/***************************************
*             Registers
***************************************/

/* Register to enable or disable the clock */
#define CyAdcMasterClock_CLKEN              (* (reg8 *) CyAdcMasterClock__PM_ACT_CFG)
#define CyAdcMasterClock_CLKEN_PTR          ((reg8 *) CyAdcMasterClock__PM_ACT_CFG)

/* Register to enable or disable the clock */
#define CyAdcMasterClock_CLKSTBY            (* (reg8 *) CyAdcMasterClock__PM_STBY_CFG)
#define CyAdcMasterClock_CLKSTBY_PTR        ((reg8 *) CyAdcMasterClock__PM_STBY_CFG)

/* Clock LSB divider configuration register. */
#define CyAdcMasterClock_DIV_LSB            (* (reg8 *) CyAdcMasterClock__CFG0)
#define CyAdcMasterClock_DIV_LSB_PTR        ((reg8 *) CyAdcMasterClock__CFG0)
#define CyAdcMasterClock_DIV_PTR            ((reg16 *) CyAdcMasterClock__CFG0)

/* Clock MSB divider configuration register. */
#define CyAdcMasterClock_DIV_MSB            (* (reg8 *) CyAdcMasterClock__CFG1)
#define CyAdcMasterClock_DIV_MSB_PTR        ((reg8 *) CyAdcMasterClock__CFG1)

/* Mode and source configuration register */
#define CyAdcMasterClock_MOD_SRC            (* (reg8 *) CyAdcMasterClock__CFG2)
#define CyAdcMasterClock_MOD_SRC_PTR        ((reg8 *) CyAdcMasterClock__CFG2)

#if defined(CyAdcMasterClock__CFG3)
/* Analog clock phase configuration register */
#define CyAdcMasterClock_PHASE              (* (reg8 *) CyAdcMasterClock__CFG3)
#define CyAdcMasterClock_PHASE_PTR          ((reg8 *) CyAdcMasterClock__CFG3)
#endif /* defined(CyAdcMasterClock__CFG3) */


/**************************************
*       Register Constants
**************************************/

/* Power manager register masks */
#define CyAdcMasterClock_CLKEN_MASK         CyAdcMasterClock__PM_ACT_MSK
#define CyAdcMasterClock_CLKSTBY_MASK       CyAdcMasterClock__PM_STBY_MSK

/* CFG2 field masks */
#define CyAdcMasterClock_SRC_SEL_MSK        CyAdcMasterClock__CFG2_SRC_SEL_MASK
#define CyAdcMasterClock_MODE_MASK          (~(CyAdcMasterClock_SRC_SEL_MSK))

#if defined(CyAdcMasterClock__CFG3)
/* CFG3 phase mask */
#define CyAdcMasterClock_PHASE_MASK         CyAdcMasterClock__CFG3_PHASE_DLY_MASK
#endif /* defined(CyAdcMasterClock__CFG3) */

#endif /* CY_CLOCK_CyAdcMasterClock_H */


/* [] END OF FILE */
