/*******************************************************************************
* File Name: CyAdcSynchroClock.h
* Version 2.20
*
*  Description:
*   Provides the function and constant definitions for the clock component.
*
*  Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CLOCK_CyAdcSynchroClock_H)
#define CY_CLOCK_CyAdcSynchroClock_H

#include <cytypes.h>
#include <cyfitter.h>


/***************************************
* Conditional Compilation Parameters
***************************************/

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component cy_clock_v2_20 requires cy_boot v3.0 or later
#endif /* (CY_PSOC5LP) */


/***************************************
*        Function Prototypes
***************************************/

void CyAdcSynchroClock_Start(void) ;
void CyAdcSynchroClock_Stop(void) ;

#if(CY_PSOC3 || CY_PSOC5LP)
void CyAdcSynchroClock_StopBlock(void) ;
#endif /* (CY_PSOC3 || CY_PSOC5LP) */

void CyAdcSynchroClock_StandbyPower(uint8 state) ;
void CyAdcSynchroClock_SetDividerRegister(uint16 clkDivider, uint8 restart) 
                                ;
uint16 CyAdcSynchroClock_GetDividerRegister(void) ;
void CyAdcSynchroClock_SetModeRegister(uint8 modeBitMask) ;
void CyAdcSynchroClock_ClearModeRegister(uint8 modeBitMask) ;
uint8 CyAdcSynchroClock_GetModeRegister(void) ;
void CyAdcSynchroClock_SetSourceRegister(uint8 clkSource) ;
uint8 CyAdcSynchroClock_GetSourceRegister(void) ;
#if defined(CyAdcSynchroClock__CFG3)
void CyAdcSynchroClock_SetPhaseRegister(uint8 clkPhase) ;
uint8 CyAdcSynchroClock_GetPhaseRegister(void) ;
#endif /* defined(CyAdcSynchroClock__CFG3) */

#define CyAdcSynchroClock_Enable()                       CyAdcSynchroClock_Start()
#define CyAdcSynchroClock_Disable()                      CyAdcSynchroClock_Stop()
#define CyAdcSynchroClock_SetDivider(clkDivider)         CyAdcSynchroClock_SetDividerRegister(clkDivider, 1u)
#define CyAdcSynchroClock_SetDividerValue(clkDivider)    CyAdcSynchroClock_SetDividerRegister((clkDivider) - 1u, 1u)
#define CyAdcSynchroClock_SetMode(clkMode)               CyAdcSynchroClock_SetModeRegister(clkMode)
#define CyAdcSynchroClock_SetSource(clkSource)           CyAdcSynchroClock_SetSourceRegister(clkSource)
#if defined(CyAdcSynchroClock__CFG3)
#define CyAdcSynchroClock_SetPhase(clkPhase)             CyAdcSynchroClock_SetPhaseRegister(clkPhase)
#define CyAdcSynchroClock_SetPhaseValue(clkPhase)        CyAdcSynchroClock_SetPhaseRegister((clkPhase) + 1u)
#endif /* defined(CyAdcSynchroClock__CFG3) */


/***************************************
*             Registers
***************************************/

/* Register to enable or disable the clock */
#define CyAdcSynchroClock_CLKEN              (* (reg8 *) CyAdcSynchroClock__PM_ACT_CFG)
#define CyAdcSynchroClock_CLKEN_PTR          ((reg8 *) CyAdcSynchroClock__PM_ACT_CFG)

/* Register to enable or disable the clock */
#define CyAdcSynchroClock_CLKSTBY            (* (reg8 *) CyAdcSynchroClock__PM_STBY_CFG)
#define CyAdcSynchroClock_CLKSTBY_PTR        ((reg8 *) CyAdcSynchroClock__PM_STBY_CFG)

/* Clock LSB divider configuration register. */
#define CyAdcSynchroClock_DIV_LSB            (* (reg8 *) CyAdcSynchroClock__CFG0)
#define CyAdcSynchroClock_DIV_LSB_PTR        ((reg8 *) CyAdcSynchroClock__CFG0)
#define CyAdcSynchroClock_DIV_PTR            ((reg16 *) CyAdcSynchroClock__CFG0)

/* Clock MSB divider configuration register. */
#define CyAdcSynchroClock_DIV_MSB            (* (reg8 *) CyAdcSynchroClock__CFG1)
#define CyAdcSynchroClock_DIV_MSB_PTR        ((reg8 *) CyAdcSynchroClock__CFG1)

/* Mode and source configuration register */
#define CyAdcSynchroClock_MOD_SRC            (* (reg8 *) CyAdcSynchroClock__CFG2)
#define CyAdcSynchroClock_MOD_SRC_PTR        ((reg8 *) CyAdcSynchroClock__CFG2)

#if defined(CyAdcSynchroClock__CFG3)
/* Analog clock phase configuration register */
#define CyAdcSynchroClock_PHASE              (* (reg8 *) CyAdcSynchroClock__CFG3)
#define CyAdcSynchroClock_PHASE_PTR          ((reg8 *) CyAdcSynchroClock__CFG3)
#endif /* defined(CyAdcSynchroClock__CFG3) */


/**************************************
*       Register Constants
**************************************/

/* Power manager register masks */
#define CyAdcSynchroClock_CLKEN_MASK         CyAdcSynchroClock__PM_ACT_MSK
#define CyAdcSynchroClock_CLKSTBY_MASK       CyAdcSynchroClock__PM_STBY_MSK

/* CFG2 field masks */
#define CyAdcSynchroClock_SRC_SEL_MSK        CyAdcSynchroClock__CFG2_SRC_SEL_MASK
#define CyAdcSynchroClock_MODE_MASK          (~(CyAdcSynchroClock_SRC_SEL_MSK))

#if defined(CyAdcSynchroClock__CFG3)
/* CFG3 phase mask */
#define CyAdcSynchroClock_PHASE_MASK         CyAdcSynchroClock__CFG3_PHASE_DLY_MASK
#endif /* defined(CyAdcSynchroClock__CFG3) */

#endif /* CY_CLOCK_CyAdcSynchroClock_H */


/* [] END OF FILE */
