/***************************************************************************//**
* \file Usb_audio.h
* \version 3.20
*
* \brief
*  This file provides function prototypes and constants for the USBFS component 
*  Audio class.
*
* Related Document:
*  Universal Serial Bus Device Class Definition for Audio Devices Release 1.0
*
********************************************************************************
* \copyright
* Copyright 2008-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_USBFS_Usb_audio_H)
#define CY_USBFS_Usb_audio_H

#include "Usb.h"


/***************************************
* Custom Declarations
***************************************/

/* `#START CUSTOM_CONSTANTS` Place your declaration here */

/* `#END` */


/***************************************
*  Constants for Usb_audio API.
***************************************/

/* Audio Class-Specific Request Codes (AUDIO Table A-9) */
#define Usb_REQUEST_CODE_UNDEFINED     (0x00u)
#define Usb_SET_CUR                    (0x01u)
#define Usb_GET_CUR                    (0x81u)
#define Usb_SET_MIN                    (0x02u)
#define Usb_GET_MIN                    (0x82u)
#define Usb_SET_MAX                    (0x03u)
#define Usb_GET_MAX                    (0x83u)
#define Usb_SET_RES                    (0x04u)
#define Usb_GET_RES                    (0x84u)
#define Usb_SET_MEM                    (0x05u)
#define Usb_GET_MEM                    (0x85u)
#define Usb_GET_STAT                   (0xFFu)

/* point Control Selectors (AUDIO Table A-19) */
#define Usb_EP_CONTROL_UNDEFINED       (0x00u)
#define Usb_SAMPLING_FREQ_CONTROL      (0x01u)
#define Usb_PITCH_CONTROL              (0x02u)

/* Feature Unit Control Selectors (AUDIO Table A-11) */
#define Usb_FU_CONTROL_UNDEFINED       (0x00u)
#define Usb_MUTE_CONTROL               (0x01u)
#define Usb_VOLUME_CONTROL             (0x02u)
#define Usb_BASS_CONTROL               (0x03u)
#define Usb_MID_CONTROL                (0x04u)
#define Usb_TREBLE_CONTROL             (0x05u)
#define Usb_GRAPHIC_EQUALIZER_CONTROL  (0x06u)
#define Usb_AUTOMATIC_GAIN_CONTROL     (0x07u)
#define Usb_DELAY_CONTROL              (0x08u)
#define Usb_BASS_BOOST_CONTROL         (0x09u)
#define Usb_LOUDNESS_CONTROL           (0x0Au)

#define Usb_SAMPLE_FREQ_LEN            (3u)
#define Usb_VOLUME_LEN                 (2u)

#if !defined(USER_SUPPLIED_DEFAULT_VOLUME_VALUE)
    #define Usb_VOL_MIN_MSB            (0x80u)
    #define Usb_VOL_MIN_LSB            (0x01u)
    #define Usb_VOL_MAX_MSB            (0x7Fu)
    #define Usb_VOL_MAX_LSB            (0xFFu)
    #define Usb_VOL_RES_MSB            (0x00u)
    #define Usb_VOL_RES_LSB            (0x01u)
#endif /* USER_SUPPLIED_DEFAULT_VOLUME_VALUE */


/***************************************
* External data references
***************************************/
/**
* \addtogroup group_audio
* @{
*/
extern volatile uint8 Usb_currentSampleFrequency[Usb_MAX_EP][Usb_SAMPLE_FREQ_LEN];
extern volatile uint8 Usb_frequencyChanged;
extern volatile uint8 Usb_currentMute;
extern volatile uint8 Usb_currentVolume[Usb_VOLUME_LEN];
/** @} audio */

extern volatile uint8 Usb_minimumVolume[Usb_VOLUME_LEN];
extern volatile uint8 Usb_maximumVolume[Usb_VOLUME_LEN];
extern volatile uint8 Usb_resolutionVolume[Usb_VOLUME_LEN];

#endif /*  CY_USBFS_Usb_audio_H */


/* [] END OF FILE */
