/***************************************************************************//**
* \file Usb_cdc.h
* \version 3.20
*
* \brief
*  This file provides function prototypes and constants for the USBFS component 
*  CDC class.
*
* Related Document:
*  Universal Serial Bus Class Definitions for Communication Devices Version 1.1
*
********************************************************************************
* \copyright
* Copyright 2012-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_USBFS_Usb_cdc_H)
#define CY_USBFS_Usb_cdc_H

#include "Usb.h"


/*******************************************************************************
* Prototypes of the Usb_cdc API.
*******************************************************************************/
/**
* \addtogroup group_cdc
* @{
*/
#if (Usb_ENABLE_CDC_CLASS_API != 0u)
    uint8 Usb_CDC_Init(void)            ;
    void Usb_PutData(const uint8* pData, uint16 length) ;
    void Usb_PutString(const char8 string[])            ;
    void Usb_PutChar(char8 txDataByte) ;
    void Usb_PutCRLF(void)             ;
    uint16 Usb_GetCount(void)          ;
    uint8  Usb_CDCIsReady(void)        ;
    uint8  Usb_DataIsReady(void)       ;
    uint16 Usb_GetData(uint8* pData, uint16 length)     ;
    uint16 Usb_GetAll(uint8* pData)    ;
    uint8  Usb_GetChar(void)           ;
    uint8  Usb_IsLineChanged(void)     ;
    uint32 Usb_GetDTERate(void)        ;
    uint8  Usb_GetCharFormat(void)     ;
    uint8  Usb_GetParityType(void)     ;
    uint8  Usb_GetDataBits(void)       ;
    uint16 Usb_GetLineControl(void)    ;
    void Usb_SendSerialState (uint16 serialState) ;
    uint16 Usb_GetSerialState (void)   ;
    void Usb_SetComPort (uint8 comNumber) ;
    uint8 Usb_GetComPort (void)        ;
    uint8 Usb_NotificationIsReady(void) ;

#endif  /* (Usb_ENABLE_CDC_CLASS_API) */
/** @} cdc */

/*******************************************************************************
*  Constants for Usb_cdc API.
*******************************************************************************/

/* CDC Class-Specific Request Codes (CDC ver 1.2 Table 19) */
#define Usb_CDC_SET_LINE_CODING        (0x20u)
#define Usb_CDC_GET_LINE_CODING        (0x21u)
#define Usb_CDC_SET_CONTROL_LINE_STATE (0x22u)

/*PSTN Subclass Specific Notifications (CDC ver 1.2 Table 30)*/
#define Usb_SERIAL_STATE               (0x20u)

#define Usb_LINE_CODING_CHANGED        (0x01u)
#define Usb_LINE_CONTROL_CHANGED       (0x02u)

#define Usb_1_STOPBIT                  (0x00u)
#define Usb_1_5_STOPBITS               (0x01u)
#define Usb_2_STOPBITS                 (0x02u)

#define Usb_PARITY_NONE                (0x00u)
#define Usb_PARITY_ODD                 (0x01u)
#define Usb_PARITY_EVEN                (0x02u)
#define Usb_PARITY_MARK                (0x03u)
#define Usb_PARITY_SPACE               (0x04u)

#define Usb_LINE_CODING_SIZE           (0x07u)
#define Usb_LINE_CODING_RATE           (0x00u)
#define Usb_LINE_CODING_STOP_BITS      (0x04u)
#define Usb_LINE_CODING_PARITY         (0x05u)
#define Usb_LINE_CODING_DATA_BITS      (0x06u)

#define Usb_LINE_CONTROL_DTR           (0x01u)
#define Usb_LINE_CONTROL_RTS           (0x02u)

#define Usb_MAX_MULTI_COM_NUM          (2u) 

#define Usb_COM_PORT1                  (0u) 
#define Usb_COM_PORT2                  (1u) 

#define Usb_SUCCESS                    (0u)
#define Usb_FAILURE                    (1u)

#define Usb_SERIAL_STATE_SIZE          (10u)

/* SerialState constants*/
#define Usb_SERIAL_STATE_REQUEST_TYPE  (0xA1u)
#define Usb_SERIAL_STATE_LENGTH        (0x2u)

/*******************************************************************************
* External data references
*******************************************************************************/
/**
* \addtogroup group_cdc
* @{
*/
extern volatile uint8  Usb_linesCoding[Usb_MAX_MULTI_COM_NUM][Usb_LINE_CODING_SIZE];
extern volatile uint8  Usb_linesChanged[Usb_MAX_MULTI_COM_NUM];
extern volatile uint16 Usb_linesControlBitmap[Usb_MAX_MULTI_COM_NUM];
extern volatile uint16 Usb_serialStateBitmap[Usb_MAX_MULTI_COM_NUM];
extern volatile uint8  Usb_cdcDataInEp[Usb_MAX_MULTI_COM_NUM];
extern volatile uint8  Usb_cdcDataOutEp[Usb_MAX_MULTI_COM_NUM];
extern volatile uint8  Usb_cdcCommInInterruptEp[Usb_MAX_MULTI_COM_NUM];
/** @} cdc */

/*******************************************************************************
* The following code is DEPRECATED and
* must not be used.
*******************************************************************************/


#define Usb_lineCoding             Usb_linesCoding[0]
#define Usb_lineChanged            Usb_linesChanged[0]
#define Usb_lineControlBitmap      Usb_linesControlBitmap[0]
#define Usb_cdc_data_in_ep         Usb_cdcDataInEp[0]
#define Usb_cdc_data_out_ep        Usb_cdcDataOutEp[0]

#endif /* (CY_USBFS_Usb_cdc_H) */


/* [] END OF FILE */
