/***************************************************************************//**
* \file .h
* \version 3.20
*
* \brief
*  This file provides private function prototypes and constants for the 
*  USBFS component. It is not intended to be used in the user project.
*
********************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_USBFS_Usb_pvt_H)
#define CY_USBFS_Usb_pvt_H

#include "Usb.h"
   
#ifdef Usb_ENABLE_AUDIO_CLASS
    #include "Usb_audio.h"
#endif /* Usb_ENABLE_AUDIO_CLASS */

#ifdef Usb_ENABLE_CDC_CLASS
    #include "Usb_cdc.h"
#endif /* Usb_ENABLE_CDC_CLASS */

#if (Usb_ENABLE_MIDI_CLASS)
    #include "Usb_midi.h"
#endif /* (Usb_ENABLE_MIDI_CLASS) */

#if (Usb_ENABLE_MSC_CLASS)
    #include "Usb_msc.h"
#endif /* (Usb_ENABLE_MSC_CLASS) */

#if (Usb_EP_MANAGEMENT_DMA)
    #if (CY_PSOC4)
        #include <CyDMA.h>
    #else
        #include <CyDmac.h>
        #if ((Usb_EP_MANAGEMENT_DMA_AUTO) && (Usb_EP_DMA_AUTO_OPT == 0u))
            #include "Usb_EP_DMA_Done_isr.h"
            #include "Usb_EP8_DMA_Done_SR.h"
            #include "Usb_EP17_DMA_Done_SR.h"
        #endif /* ((Usb_EP_MANAGEMENT_DMA_AUTO) && (Usb_EP_DMA_AUTO_OPT == 0u)) */
    #endif /* (CY_PSOC4) */
#endif /* (Usb_EP_MANAGEMENT_DMA) */

#if (Usb_DMA1_ACTIVE)
    #include "Usb_ep1_dma.h"
    #define Usb_EP1_DMA_CH     (Usb_ep1_dma_CHANNEL)
#endif /* (Usb_DMA1_ACTIVE) */

#if (Usb_DMA2_ACTIVE)
    #include "Usb_ep2_dma.h"
    #define Usb_EP2_DMA_CH     (Usb_ep2_dma_CHANNEL)
#endif /* (Usb_DMA2_ACTIVE) */

#if (Usb_DMA3_ACTIVE)
    #include "Usb_ep3_dma.h"
    #define Usb_EP3_DMA_CH     (Usb_ep3_dma_CHANNEL)
#endif /* (Usb_DMA3_ACTIVE) */

#if (Usb_DMA4_ACTIVE)
    #include "Usb_ep4_dma.h"
    #define Usb_EP4_DMA_CH     (Usb_ep4_dma_CHANNEL)
#endif /* (Usb_DMA4_ACTIVE) */

#if (Usb_DMA5_ACTIVE)
    #include "Usb_ep5_dma.h"
    #define Usb_EP5_DMA_CH     (Usb_ep5_dma_CHANNEL)
#endif /* (Usb_DMA5_ACTIVE) */

#if (Usb_DMA6_ACTIVE)
    #include "Usb_ep6_dma.h"
    #define Usb_EP6_DMA_CH     (Usb_ep6_dma_CHANNEL)
#endif /* (Usb_DMA6_ACTIVE) */

#if (Usb_DMA7_ACTIVE)
    #include "Usb_ep7_dma.h"
    #define Usb_EP7_DMA_CH     (Usb_ep7_dma_CHANNEL)
#endif /* (Usb_DMA7_ACTIVE) */

#if (Usb_DMA8_ACTIVE)
    #include "Usb_ep8_dma.h"
    #define Usb_EP8_DMA_CH     (Usb_ep8_dma_CHANNEL)
#endif /* (Usb_DMA8_ACTIVE) */


/***************************************
*     Private Variables
***************************************/

/* Generated external references for descriptors. */
extern const uint8 CYCODE Usb_DEVICE0_DESCR[18u];
extern const uint8 CYCODE Usb_DEVICE0_CONFIGURATION0_DESCR[67u];
extern const T_Usb_EP_SETTINGS_BLOCK CYCODE Usb_DEVICE0_CONFIGURATION0_EP_SETTINGS_TABLE[3u];
extern const uint8 CYCODE Usb_DEVICE0_CONFIGURATION0_INTERFACE_CLASS[2u];
extern const T_Usb_LUT CYCODE Usb_DEVICE0_CONFIGURATION0_TABLE[5u];
extern const T_Usb_LUT CYCODE Usb_DEVICE0_TABLE[3u];
extern const T_Usb_LUT CYCODE Usb_TABLE[1u];
extern const uint8 CYCODE Usb_SN_STRING_DESCRIPTOR[2];
extern const uint8 CYCODE Usb_STRING_DESCRIPTORS[185u];


extern const uint8 CYCODE Usb_MSOS_DESCRIPTOR[Usb_MSOS_DESCRIPTOR_LENGTH];
extern const uint8 CYCODE Usb_MSOS_CONFIGURATION_DESCR[Usb_MSOS_CONF_DESCR_LENGTH];
#if defined(Usb_ENABLE_IDSN_STRING)
    extern uint8 Usb_idSerialNumberStringDescriptor[Usb_IDSN_DESCR_LENGTH];
#endif /* (Usb_ENABLE_IDSN_STRING) */

extern volatile uint8 Usb_interfaceNumber;
extern volatile uint8 Usb_interfaceSetting[Usb_MAX_INTERFACES_NUMBER];
extern volatile uint8 Usb_interfaceSettingLast[Usb_MAX_INTERFACES_NUMBER];
extern volatile uint8 Usb_deviceAddress;
extern volatile uint8 Usb_interfaceStatus[Usb_MAX_INTERFACES_NUMBER];
extern const uint8 CYCODE *Usb_interfaceClass;

extern volatile T_Usb_EP_CTL_BLOCK Usb_EP[Usb_MAX_EP];
extern volatile T_Usb_TD Usb_currentTD;

#if (Usb_EP_MANAGEMENT_DMA)
    #if (CY_PSOC4)
        extern const uint8 Usb_DmaChan[Usb_MAX_EP];
    #else
        extern uint8 Usb_DmaChan[Usb_MAX_EP];
        extern uint8 Usb_DmaTd  [Usb_MAX_EP];
    #endif /* (CY_PSOC4) */
#endif /* (Usb_EP_MANAGEMENT_DMA) */

#if (Usb_EP_MANAGEMENT_DMA_AUTO)
#if (CY_PSOC4)
    extern uint8  Usb_DmaEpBurstCnt   [Usb_MAX_EP];
    extern uint8  Usb_DmaEpLastBurstEl[Usb_MAX_EP];

    extern uint8  Usb_DmaEpBurstCntBackup  [Usb_MAX_EP];
    extern uint32 Usb_DmaEpBufferAddrBackup[Usb_MAX_EP];
    
    extern const uint8 Usb_DmaReqOut     [Usb_MAX_EP];    
    extern const uint8 Usb_DmaBurstEndOut[Usb_MAX_EP];
#else
    #if (Usb_EP_DMA_AUTO_OPT == 0u)
        extern uint8 Usb_DmaNextTd[Usb_MAX_EP];
        extern volatile uint16 Usb_inLength [Usb_MAX_EP];
        extern volatile uint8  Usb_inBufFull[Usb_MAX_EP];
        extern const uint8 Usb_epX_TD_TERMOUT_EN[Usb_MAX_EP];
        extern const uint8 *Usb_inDataPointer[Usb_MAX_EP];
    #endif /* (Usb_EP_DMA_AUTO_OPT == 0u) */
#endif /* CY_PSOC4 */
#endif /* (Usb_EP_MANAGEMENT_DMA_AUTO) */

extern volatile uint8 Usb_ep0Toggle;
extern volatile uint8 Usb_lastPacketSize;
extern volatile uint8 Usb_ep0Mode;
extern volatile uint8 Usb_ep0Count;
extern volatile uint16 Usb_transferByteCount;


/***************************************
*     Private Function Prototypes
***************************************/
void  Usb_ReInitComponent(void)            ;
void  Usb_HandleSetup(void)                ;
void  Usb_HandleIN(void)                   ;
void  Usb_HandleOUT(void)                  ;
void  Usb_LoadEP0(void)                    ;
uint8 Usb_InitControlRead(void)            ;
uint8 Usb_InitControlWrite(void)           ;
void  Usb_ControlReadDataStage(void)       ;
void  Usb_ControlReadStatusStage(void)     ;
void  Usb_ControlReadPrematureStatus(void) ;
uint8 Usb_InitControlWrite(void)           ;
uint8 Usb_InitZeroLengthControlTransfer(void) ;
void  Usb_ControlWriteDataStage(void)      ;
void  Usb_ControlWriteStatusStage(void)    ;
void  Usb_ControlWritePrematureStatus(void);
uint8 Usb_InitNoDataControlTransfer(void)  ;
void  Usb_NoDataControlStatusStage(void)   ;
void  Usb_InitializeStatusBlock(void)      ;
void  Usb_UpdateStatusBlock(uint8 completionCode) ;
uint8 Usb_DispatchClassRqst(void)          ;

void Usb_Config(uint8 clearAltSetting) ;
void Usb_ConfigAltChanged(void)        ;
void Usb_ConfigReg(void)               ;
void Usb_EpStateInit(void)             ;


const T_Usb_LUT CYCODE *Usb_GetConfigTablePtr(uint8 confIndex);
const T_Usb_LUT CYCODE *Usb_GetDeviceTablePtr(void)           ;
#if (Usb_BOS_ENABLE)
    const T_Usb_LUT CYCODE *Usb_GetBOSPtr(void)               ;
#endif /* (Usb_BOS_ENABLE) */
const uint8 CYCODE *Usb_GetInterfaceClassTablePtr(void)                    ;
uint8 Usb_ClearEndpointHalt(void)                                          ;
uint8 Usb_SetEndpointHalt(void)                                            ;
uint8 Usb_ValidateAlternateSetting(void)                                   ;

void Usb_SaveConfig(void)      ;
void Usb_RestoreConfig(void)   ;

#if (CY_PSOC3 || CY_PSOC5LP)
    #if (Usb_EP_MANAGEMENT_DMA_AUTO && (Usb_EP_DMA_AUTO_OPT == 0u))
        void Usb_LoadNextInEP(uint8 epNumber, uint8 mode)  ;
    #endif /* (Usb_EP_MANAGEMENT_DMA_AUTO && (Usb_EP_DMA_AUTO_OPT == 0u)) */
#endif /* (CY_PSOC3 || CY_PSOC5LP) */

#if defined(Usb_ENABLE_IDSN_STRING)
    void Usb_ReadDieID(uint8 descr[])  ;
#endif /* Usb_ENABLE_IDSN_STRING */

#if defined(Usb_ENABLE_HID_CLASS)
    uint8 Usb_DispatchHIDClassRqst(void) ;
#endif /* (Usb_ENABLE_HID_CLASS) */

#if defined(Usb_ENABLE_AUDIO_CLASS)
    uint8 Usb_DispatchAUDIOClassRqst(void) ;
#endif /* (Usb_ENABLE_AUDIO_CLASS) */

#if defined(Usb_ENABLE_CDC_CLASS)
    uint8 Usb_DispatchCDCClassRqst(void) ;
#endif /* (Usb_ENABLE_CDC_CLASS) */

#if (Usb_ENABLE_MSC_CLASS)
    #if (Usb_HANDLE_MSC_REQUESTS)
        uint8 Usb_DispatchMSCClassRqst(void) ;
    #endif /* (Usb_HANDLE_MSC_REQUESTS) */
#endif /* (Usb_ENABLE_MSC_CLASS */

CY_ISR_PROTO(Usb_EP_0_ISR);
CY_ISR_PROTO(Usb_BUS_RESET_ISR);

#if (Usb_SOF_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_SOF_ISR);
#endif /* (Usb_SOF_ISR_ACTIVE) */

#if (Usb_EP1_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_1_ISR);
#endif /* (Usb_EP1_ISR_ACTIVE) */

#if (Usb_EP2_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_2_ISR);
#endif /* (Usb_EP2_ISR_ACTIVE) */

#if (Usb_EP3_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_3_ISR);
#endif /* (Usb_EP3_ISR_ACTIVE) */

#if (Usb_EP4_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_4_ISR);
#endif /* (Usb_EP4_ISR_ACTIVE) */

#if (Usb_EP5_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_5_ISR);
#endif /* (Usb_EP5_ISR_ACTIVE) */

#if (Usb_EP6_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_6_ISR);
#endif /* (Usb_EP6_ISR_ACTIVE) */

#if (Usb_EP7_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_7_ISR);
#endif /* (Usb_EP7_ISR_ACTIVE) */

#if (Usb_EP8_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_EP_8_ISR);
#endif /* (Usb_EP8_ISR_ACTIVE) */

#if (Usb_EP_MANAGEMENT_DMA)
    CY_ISR_PROTO(Usb_ARB_ISR);
#endif /* (Usb_EP_MANAGEMENT_DMA) */

#if (Usb_DP_ISR_ACTIVE)
    CY_ISR_PROTO(Usb_DP_ISR);
#endif /* (Usb_DP_ISR_ACTIVE) */

#if (CY_PSOC4)
    CY_ISR_PROTO(Usb_INTR_HI_ISR);
    CY_ISR_PROTO(Usb_INTR_MED_ISR);
    CY_ISR_PROTO(Usb_INTR_LO_ISR);
    #if (Usb_LPM_ACTIVE)
        CY_ISR_PROTO(Usb_LPM_ISR);
    #endif /* (Usb_LPM_ACTIVE) */
#endif /* (CY_PSOC4) */

#if (Usb_EP_MANAGEMENT_DMA_AUTO)
#if (CY_PSOC4)
    #if (Usb_DMA1_ACTIVE)
        void Usb_EP1_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA1_ACTIVE) */

    #if (Usb_DMA2_ACTIVE)
        void Usb_EP2_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA2_ACTIVE) */

    #if (Usb_DMA3_ACTIVE)
        void Usb_EP3_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA3_ACTIVE) */

    #if (Usb_DMA4_ACTIVE)
        void Usb_EP4_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA4_ACTIVE) */

    #if (Usb_DMA5_ACTIVE)
        void Usb_EP5_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA5_ACTIVE) */

    #if (Usb_DMA6_ACTIVE)
        void Usb_EP6_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA6_ACTIVE) */

    #if (Usb_DMA7_ACTIVE)
        void Usb_EP7_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA7_ACTIVE) */

    #if (Usb_DMA8_ACTIVE)
        void Usb_EP8_DMA_DONE_ISR(void);
    #endif /* (Usb_DMA8_ACTIVE) */

#else
    #if (Usb_EP_DMA_AUTO_OPT == 0u)
        CY_ISR_PROTO(Usb_EP_DMA_DONE_ISR);
    #endif /* (Usb_EP_DMA_AUTO_OPT == 0u) */
#endif /* (CY_PSOC4) */
#endif /* (Usb_EP_MANAGEMENT_DMA_AUTO) */


/***************************************
*         Request Handlers
***************************************/

uint8 Usb_HandleStandardRqst(void) ;
uint8 Usb_DispatchClassRqst(void)  ;
uint8 Usb_HandleVendorRqst(void)   ;


/***************************************
*    HID Internal references
***************************************/

#if defined(Usb_ENABLE_HID_CLASS)
    void Usb_FindReport(void)            ;
    void Usb_FindReportDescriptor(void)  ;
    void Usb_FindHidClassDecriptor(void) ;
#endif /* Usb_ENABLE_HID_CLASS */


/***************************************
*    MIDI Internal references
***************************************/

#if defined(Usb_ENABLE_MIDI_STREAMING)
    void Usb_MIDI_IN_EP_Service(void)  ;
#endif /* (Usb_ENABLE_MIDI_STREAMING) */


/***************************************
*    CDC Internal references
***************************************/

#if defined(Usb_ENABLE_CDC_CLASS)

    typedef struct
    {
        uint8  bRequestType;
        uint8  bNotification;
        uint8  wValue;
        uint8  wValueMSB;
        uint8  wIndex;
        uint8  wIndexMSB;
        uint8  wLength;
        uint8  wLengthMSB;
        uint8  wSerialState;
        uint8  wSerialStateMSB;
    } t_Usb_cdc_notification;

    uint8 Usb_GetInterfaceComPort(uint8 interface) ;
    uint8 Usb_Cdc_EpInit( const T_Usb_EP_SETTINGS_BLOCK CYCODE *pEP, uint8 epNum, uint8 cdcComNums) ;

    extern volatile uint8  Usb_cdc_dataInEpList[Usb_MAX_MULTI_COM_NUM];
    extern volatile uint8  Usb_cdc_dataOutEpList[Usb_MAX_MULTI_COM_NUM];
    extern volatile uint8  Usb_cdc_commInEpList[Usb_MAX_MULTI_COM_NUM];
#endif /* (Usb_ENABLE_CDC_CLASS) */


#endif /* CY_USBFS_Usb_pvt_H */


/* [] END OF FILE */
