/***************************************************************************//**
* \file Usb_vnd.c
* \version 3.20
*
* \brief
*  This file contains the  USB vendor request handler.
*
********************************************************************************
* \copyright
* Copyright 2008-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Usb_pvt.h"
#include "cyapicallbacks.h"

#if(Usb_EXTERN_VND == Usb_FALSE)

/***************************************
* Vendor Specific Declarations
***************************************/

/* `#START VENDOR_SPECIFIC_DECLARATIONS` Place your declaration here */

/* `#END` */


/*******************************************************************************
* Function Name: Usb_HandleVendorRqst
****************************************************************************//**
*
*  This routine provide users with a method to implement vendor specific
*  requests.
*
*  To implement vendor specific requests, add your code in this function to
*  decode and disposition the request.  If the request is handled, your code
*  must set the variable "requestHandled" to TRUE, indicating that the
*  request has been handled.
*
* \return
*  requestHandled.
*
* \reentrant
*  No.
*
*******************************************************************************/
uint8 Usb_HandleVendorRqst(void) 
{
    uint8 requestHandled = Usb_FALSE;

    /* Check request direction: D2H or H2D. */
    if (0u != (Usb_bmRequestTypeReg & Usb_RQST_DIR_D2H))
    {
        /* Handle direction from device to host. */
        
        switch (Usb_bRequestReg)
        {
            case Usb_GET_EXTENDED_CONFIG_DESCRIPTOR:
            #if defined(Usb_ENABLE_MSOS_STRING)
                Usb_currentTD.pData = (volatile uint8 *) &Usb_MSOS_CONFIGURATION_DESCR[0u];
                Usb_currentTD.count = Usb_MSOS_CONFIGURATION_DESCR[0u];
                requestHandled  = Usb_InitControlRead();
            #endif /* (Usb_ENABLE_MSOS_STRING) */
                break;
            
            default:
                break;
        }
    }

    /* `#START VENDOR_SPECIFIC_CODE` Place your vendor specific request here */

    /* `#END` */

#ifdef Usb_HANDLE_VENDOR_RQST_CALLBACK
    if (Usb_FALSE == requestHandled)
    {
        requestHandled = Usb_HandleVendorRqst_Callback();
    }
#endif /* (Usb_HANDLE_VENDOR_RQST_CALLBACK) */

    return (requestHandled);
}


/*******************************************************************************
* Additional user functions supporting Vendor Specific Requests
********************************************************************************/

/* `#START VENDOR_SPECIFIC_FUNCTIONS` Place any additional functions here */

/* `#END` */


#endif /* Usb_EXTERN_VND */


/* [] END OF FILE */
